import numpy as np
import pickle
import scipy.io as sio
import os
from sqlalchemy import null
import torch
from torch.utils.data import Dataset
from torch.utils.data import TensorDataset
from torch.utils.data import DataLoader
from sklearn.preprocessing import MinMaxScaler

NUMBER_OF_SUBJECTS = 16
EEG_CHANNELS = 32
EEG_BANDS = 4
PERIPHERAL_CHANNELS = 8
PERIPHERAL_BANDS = 8
VALENCE_FLAG = True
PSD_FLAG = False

DATA_PATH = "../DEAP/fft/"

def load_data_eeg(idx, start, end):
    if idx < 10:
        filename = "0"+str(idx)+".npz"
    else:
        filename = str(idx)+".npz"

    data = np.load(DATA_PATH + filename)
    EEG_data = data['psd'] if PSD_FLAG else data['diff_entropy']
    EEG_data = np.reshape(EEG_data, (len(EEG_data), EEG_CHANNELS*EEG_BANDS))
    EEG_data[~np.isfinite(EEG_data)] = 0
    EEG_label = data['valence_labels'] if VALENCE_FLAG else data['arousal_labels']
    i_0 = start-1
    ses1_X_train = EEG_data[i_0]
    ses1_y_train = EEG_label[i_0]
    for session in range(start,  end):
        ses1_X_train = np.vstack((ses1_X_train, EEG_data[session] ))
        ses1_y_train = np.concatenate((ses1_y_train, EEG_label[session]), axis = None)

    scaler = MinMaxScaler()
    ses1_X_train = scaler.fit(ses1_X_train).transform(ses1_X_train)
    return ses1_X_train, ses1_y_train

def load_data_eye(idx, start, end):
    if idx < 10:
        filename = "0"+str(idx)+".npz"
    else:
        filename = str(idx)+".npz"

    data = np.load(DATA_PATH + filename)
    EYE_data = data['peripheral_data']
    EYE_data = np.reshape(EYE_data, (len(EYE_data), PERIPHERAL_CHANNELS*PERIPHERAL_BANDS))
    EYE_data[~np.isfinite(EYE_data)] = 0
    EYE_label = data['valence_labels'] if VALENCE_FLAG else data['arousal_labels']
    i_0 = start-1
    ses1_X_train = EYE_data[i_0]
    ses1_y_train = EYE_label[i_0]
    for session in range(start, end):
        ses1_X_train = np.vstack((ses1_X_train, EYE_data[session] ))
        ses1_y_train = np.concatenate((ses1_y_train, EYE_label[session]), axis = None)

    scaler = MinMaxScaler()
    ses1_X_train = scaler.fit(ses1_X_train).transform(ses1_X_train)
    return ses1_X_train, ses1_y_train

def load_session_transfer_data(subject_id, test_session_id):
    if test_session_id == 1:
        data_start = 601
        data_end = 1200
        gen_start = 1
        gen_end = 450
        test_start = 451
        test_end = 600
    elif test_session_id == 2:
        data_start = 1
        data_end = 600
        gen_start = 601
        gen_end = 1050
        test_start = 1051
        test_end = 1200

    EEG_data, EEG_label = load_data_eeg(subject_id, data_start, data_end)
    EYE_data, EYE_label = load_data_eye(subject_id, data_start, data_end)
    EEG_gen_test, EEG_gen_label = load_data_eeg(subject_id, gen_start, gen_end)
    EYE_gen_test, EYE_gen_label = load_data_eye(subject_id, gen_start, gen_end)
    EEG_svc_test, EEG_svc_label = load_data_eeg(subject_id, test_start, test_end)
    EYE_svc_test, EYE_svc_label = load_data_eye(subject_id, test_start, test_end)

    return EEG_data, EEG_label, EYE_data, EYE_label, EEG_gen_test, EEG_gen_label, EYE_gen_test, EYE_gen_label, EEG_svc_test, EEG_svc_label, EYE_svc_test, EYE_svc_label

def load_subject_transfer_data(subject_id, test_session_id):
    EEG_data, EEG_label, EYE_data, EYE_label = null, null, null, null
    for i in range(1, NUMBER_OF_SUBJECTS+1):
        if i == subject_id:
            if test_session_id == 1:
                gen_start = 601
                gen_end = 1200
                test_start = 1
                test_end = 600
            elif test_session_id == 2:
                gen_start = 1
                gen_end = 600
                test_start = 601
                test_end = 1200
            EEG_gen_test, EEG_gen_label = load_data_eeg(subject_id, gen_start, gen_end)
            EYE_gen_test, EYE_gen_label = load_data_eye(subject_id, gen_start, gen_end)
            EEG_svc_test, EEG_svc_label = load_data_eeg(subject_id, test_start, test_end)
            EYE_svc_test, EYE_svc_label = load_data_eye(subject_id, test_start, test_end)
        else:
            if EEG_data is null:
                EEG_data, EEG_label = load_data_eeg(i, 1, 1200)
                EYE_data, EYE_label = load_data_eye(i, 1, 1200)
            else:
                EEG_data2, EEG_label2 = load_data_eeg(i, 1, 1200)
                EYE_data2, EYE_label2 = load_data_eye(i, 1, 1200)
                EEG_data = np.vstack((EEG_data, EEG_data2))
                EEG_label = np.concatenate((EEG_label, EEG_label2), axis = 0)
                EYE_data = np.vstack((EYE_data, EYE_data2))
                EYE_label = np.concatenate((EYE_label, EYE_label2), axis = 0)

    return EEG_data, EEG_label, EYE_data, EYE_label, EEG_gen_test, EEG_gen_label, EYE_gen_test, EYE_gen_label, EEG_svc_test, EEG_svc_label, EYE_svc_test, EYE_svc_label

if __name__ == '__main__':
    EEG_data, EEG_label = load_data_eeg(1, 1, 1200)
    EYE_data, EYE_label = load_data_eye(1, 1, 1200)
    print((EEG_label==EYE_label).all())
    print("EEG:",EEG_data.shape,"EYE:",EYE_data.shape," label:",EEG_label.shape)
    #train_dataset = TensorDataset(torch.FloatTensor(EEG_data), torch.FloatTensor(EYE_data), torch.LongTensor(EEG_label))
    #train_dataloader = DataLoader(train_dataset, batch_size = 20, shuffle = True)
