import numpy as np
import pandas as pd
import pickle
import matplotlib.pyplot as plt
from scipy import signal
from scipy.signal import welch
from scipy.integrate import simps
from scipy.stats import f_oneway
from scipy.stats import differential_entropy
import signalpreprocess as sp

NUM_SUBJECTS = 32
DATA_PATH = "C://Users/YOGA/Desktop/phd/datasets/deap_archive/data_preprocessed_python/"
SAVE_PATH = "C://Users/YOGA/Desktop/phd/datasets/deap_archive/psd/"
num_trials = 40
# data = [40, 40, 8064]
# labels = [40, 4]

theta_band = [4, 8]
alpha_band = [8, 12]
beta_band = [12, 30]
gamma_band = [30, 64]

theta_band = [3, 7]
alpha_band = [8, 13]
beta_band = [14, 29]
gamma_band = [30, 47]

# Function to load data from each participant file
def read_eeg_signal_from_file(filename):
    x = pickle._Unpickler(open(filename, 'rb'))
    x.encoding = 'latin1'
    p = x.load()
    return p

def bandpower(data, sf, band, window_sec=None, relative=False):
    band = np.asarray(band)
    low, high = band
    # Define window length
    if window_sec is not None:
        nperseg = window_sec * sf
    else:
        nperseg = (2 / low) * sf
    # Compute the modified periodogram (Welch)
    freqs, psd = welch(data, sf, nperseg=nperseg)
    # Frequency resolution
    freq_res = freqs[1] - freqs[0]
    # Find closest indices of band in frequency vector
    idx_band = np.logical_and(freqs >= low, freqs <= high)
    # Integral approximation of the spectrum using Simpson's rule.
    bp = simps(psd[idx_band], dx=freq_res)
    if relative:
        bp /= simps(psd, dx=freq_res)
    return bp

def frequency_bands(data, band):
    if (band == "theta"): # drownsiness, emotional connection, intuition, creativity
        bd = theta_band
    elif (band == "alpha"): # reflection, relaxation
        bd = alpha_band
    elif (band == "beta"): # concentration, problem solving, memory
        bd = beta_band
    elif (band == "gamma"): # cognition, perception, learning, multi-tasking
        bd = gamma_band
    return bandpower(data,128, bd)

def get_band_powers(data):
    power = np.zeros([4])
    bands = ["theta", "alpha", "beta", "gamma"]
    for idx, band in enumerate(bands):
        power[idx] = frequency_bands(data, band)
    return power

def calc_entropy(data, band):
    if (band == "theta"):
        bd = theta_band
    elif (band == "alpha"):
        bd = alpha_band
    elif (band == "beta"):
        bd = beta_band
    elif (band == "gamma"):
        bd = gamma_band
    
    band = np.asarray(bd)
    low, high = band
    # Define sampling frequency and time vector
    sf = 128.
    # Define window length (4 seconds)
    win = 4 * sf
    #win = (2 / low) * sf
    freqs, psd = signal.welch(data, sf, nperseg=win)
    idx_band = np.logical_and(freqs >= low, freqs <= high)
    return differential_entropy(psd[idx_band])

def get_entropy(data):
    entropy = np.zeros([4])
    bands = ["theta", "alpha", "beta", "gamma"]
    for idx, band in enumerate(bands):
        entropy[idx] = calc_entropy(data, band)
    return entropy

### Separate EEG and non-EEG data
# The dataset includes 32 EEG channels and 8 peripheral physiological channels. 
# The peripheral signals include: electrooculogram (EOG), electromyograms (EMG) of Zygomaticus and Trapezius muscles
# GSR, respiration amplitude, blood volume by plethysmograph, skin temperature.
eeg_channels = np.array(["Fp1", "AF3", "F3", "F7", "FC5", "FC1", "C3", "T7", "CP5", "CP1", "P3", "P7", "PO3", "O1", "Oz", "Pz", "Fp2", "AF4", "Fz", "F4", "F8", "FC6", "FC2", "Cz", "C4", "T8", "CP6", "CP2", "P4", "P8", "PO4", "O2"])
peripheral_channels = np.array(["hEOG", "vEOG", "zEMG", "tEMG", "GSR", "Respiration belt", "Plethysmograph", "Temperature"])


def get_eeg_features(data):
    eeg_data = []
    for i in range (len(data)):
        for j in range (len(eeg_channels)):
            eeg_data.append(data[i,j])
    eeg_data = np.reshape(eeg_data, (len(data), len(eeg_channels), len(data[0,0])))
    n_samples = eeg_data.shape[0]
    n_channels = 32
    n_band = 4
    psd = np.zeros((n_samples, n_channels, n_band))
    diff_entropy = np.zeros((n_samples, n_channels, n_band))
    for i in range(n_samples):
        for j in range(n_channels):
            psd[i,j,:] = get_band_powers(eeg_data[i,j,:])
            diff_entropy[i,j,:] = get_entropy(eeg_data[i,j,:])
    return psd, diff_entropy

def get_peripheral_features(data):
    peripheral_data = np.zeros((40, 8, 8))
    for i in range (len(data)):
        for j in range (32, len(data[0])):
            signal = data[i,j]
            m, s, nfd, nsd, mini, maxi = sp.package_deal_signal_mm(signal)
            ps = sp.power_spectrums(signal)
            ag = sp.average_gradient(signal)
            peripheral_data[i,j-32,:] = np.array([m,s,nfd,nsd, mini, maxi, ps, ag])
    return peripheral_data

def get_labels(labels):
    # 0 valence, 1 arousal, 2 dominance, 3 liking
    valence_labels = labels[:, 0] >= np.median(labels[:,0]) # valence labels
    # convert true to 1, false to 0
    valence_labels = valence_labels.astype(int) 
    arousal_labels = labels[:, 1] >= np.median(labels[:,1]) # arousal labels
    arousal_labels = arousal_labels.astype(int)
    return valence_labels, arousal_labels

if __name__ == '__main__':
    for n in range(1, NUM_SUBJECTS+1): 
        filename = ''
        if n < 10:
            filename += '0'
        filename += str(n)
        print("processing: ", filename, "......")
        file_path = DATA_PATH + "s" + filename + ".dat"
        trial = read_eeg_signal_from_file(file_path)
        labels = trial['labels']
        data = trial['data']
        # eeg features
        psd, diff_entropy = get_eeg_features(data)
        # peripheral features
        peripheral_data = get_peripheral_features(data)
        # labels
        valence_labels, arousal_labels = get_labels(labels)
        # save features as npz
        np.savez_compressed(SAVE_PATH + filename + ".npz", psd=psd, diff_entropy=diff_entropy, peripheral_data=peripheral_data, valence_labels=valence_labels, arousal_labels=arousal_labels)        
    print("Done")
