import numpy as np
import pandas as pd
import pickle
import matplotlib.pyplot as plt
from scipy import signal
from scipy.signal import welch
from scipy.integrate import simps
from scipy.stats import f_oneway
from scipy.stats import differential_entropy
import signalpreprocess as sp
from numpy.fft import fft
from math import floor
#from pyeeg import bin_power

NUM_SUBJECTS = 32
DATA_PATH = "C://Users/YOGA/Desktop/phd/datasets/deap_archive/data_preprocessed_python/"
SAVE_PATH = "C://Users/YOGA/Desktop/phd/datasets/deap_archive/fft/"
num_trials = 40
# data = [40, 40, 8064]
# labels = [40, 4]

theta_band = [4, 8]
alpha_band = [8, 12]
beta_band = [12, 30]
gamma_band = [30, 45]

#band = [4,8,12,16,25,45] #5 bands
band = [4,8,12,30,45] #4 bands

average_sec = 3 #Averaging band power of 3 sec
sample_rate = 128 #Sampling rate of 128 Hz
window_size = average_sec*sample_rate #Averaging band power of 3 sec
update_sec = 2 #Each 1 sec update once
step_size = sample_rate*update_sec #Each 1 sec update once

def bin_power(X,Band,Fs):
	C = fft(X)
	C = abs(C)
	Power = np.zeros(len(Band)-1)
	for Freq_Index in range(0,len(Band)-1):
		Freq = float(Band[Freq_Index])
		Next_Freq = float(Band[Freq_Index+1])
		Power[Freq_Index] = sum(C[floor(Freq/Fs*len(X)):floor(Next_Freq/Fs*len(X))])
	Power_Ratio = Power/sum(Power)
	return Power, Power_Ratio	

# Function to load data from each participant file
def read_eeg_signal_from_file(filename):
    x = pickle._Unpickler(open(filename, 'rb'))
    x.encoding = 'latin1'
    p = x.load()
    return p

def bandpower(data, sf, band, window_sec=None, relative=False):
    band = np.asarray(band)
    low, high = band
    # Define window length
    if window_sec is not None:
        nperseg = window_sec * sf
    else:
        nperseg = (2 / low) * sf
    # Compute the modified periodogram (Welch)
    freqs, psd = welch(data, sf, nperseg=nperseg)
    # Frequency resolution
    freq_res = freqs[1] - freqs[0]
    # Find closest indices of band in frequency vector
    idx_band = np.logical_and(freqs >= low, freqs <= high)
    # Integral approximation of the spectrum using Simpson's rule.
    bp = simps(psd[idx_band], dx=freq_res)
    if relative:
        bp /= simps(psd, dx=freq_res)
    return bp

def frequency_bands(data, band):
    if (band == "theta"): # drownsiness, emotional connection, intuition, creativity
        bd = theta_band
    elif (band == "alpha"): # reflection, relaxation
        bd = alpha_band
    elif (band == "beta"): # concentration, problem solving, memory
        bd = beta_band
    elif (band == "gamma"): # cognition, perception, learning, multi-tasking
        bd = gamma_band
    return bandpower(data,128, bd)

def get_band_powers(data):
    power = np.zeros([4])
    bands = ["theta", "alpha", "beta", "gamma"]
    for idx, band in enumerate(bands):
        power[idx] = frequency_bands(data, band)
    return power

def calc_entropy(data, band):
    if (band == "theta"):
        bd = theta_band
    elif (band == "alpha"):
        bd = alpha_band
    elif (band == "beta"):
        bd = beta_band
    elif (band == "gamma"):
        bd = gamma_band
    
    band = np.asarray(bd)
    low, high = band
    # Define sampling frequency and time vector
    sf = 128.
    # Define window length (0.5 seconds)
    win = 2*128 # 0.125 seconds
    #win = (2 / low) * sf
    freqs, psd = signal.welch(data, sf, nperseg=win)
    idx_band = np.logical_and(freqs >= low, freqs <= high)
    return differential_entropy(psd[idx_band])

def get_entropy(data):
    entropy = np.zeros([4])
    bands = ["theta", "alpha", "beta", "gamma"]
    for idx, band in enumerate(bands):
        entropy[idx] = calc_entropy(data, band)
    return entropy

### Separate EEG and non-EEG data
# The dataset includes 32 EEG channels and 8 peripheral physiological channels. 
# The peripheral signals include: electrooculogram (EOG), electromyograms (EMG) of Zygomaticus and Trapezius muscles
# GSR, respiration amplitude, blood volume by plethysmograph, skin temperature.
eeg_channels = np.array(["Fp1", "AF3", "F3", "F7", "FC5", "FC1", "C3", "T7", "CP5", "CP1", "P3", "P7", "PO3", "O1", "Oz", "Pz", "Fp2", "AF4", "Fz", "F4", "F8", "FC6", "FC2", "Cz", "C4", "T8", "CP6", "CP2", "P4", "P8", "PO4", "O2"])
peripheral_channels = np.array(["hEOG", "vEOG", "zEMG", "tEMG", "GSR", "Respiration belt", "Plethysmograph", "Temperature"])

def get_eeg_features(data):
    fft_data = []
    psd_data = []
    entropy_data = []
    for i in range (len(data)):
        data_i = data[i]
        start = 0
        while start + window_size < data_i.shape[1]:
            fft_array = []
            psd_array = []
            entropy_array = []
            for j in range (len(eeg_channels)):
                X = data_i[j, start:start+window_size] #Slice raw data over 3 sec, at interval of 1 sec
                fft_power = bin_power(X, band, sample_rate)
                fft_array = fft_array + list(fft_power[0])
                psd_array = psd_array + list(get_band_powers(X))
                entropy_array = entropy_array + list(get_entropy(X))
            fft_data.append(np.array(fft_array))
            psd_data.append(np.array(psd_array))
            entropy_data.append(np.array(entropy_array))
            start += step_size
    return np.array(fft_data), np.array(psd_data), np.array(entropy_data)

def get_peripheral_features(data):
    peripheral_data = []
    for i in range (len(data)):
        data_i = data[i]
        start = 0
        while start + window_size < data_i.shape[1]:
            peripheral_array = []
            for j in range (32, len(data[0])):
                base = data_i[j, :384]
                signal = data_i[j, start:start+window_size]
                m, s, nfd, nsd, mini, maxi = sp.package_deal_signal_mm(signal)
                ps = sp.power_spectrums(base, signal)
                ag = sp.average_gradient(signal)
                peripheral_array = peripheral_array + [m,s,nfd,nsd, mini, maxi, ps, ag]
            peripheral_data.append(np.array(peripheral_array))
            start += step_size
    return np.array(peripheral_data)

def get_labels(labels):
    sample_size = int((63-average_sec)/update_sec)
    # 0 valence, 1 arousal, 2 dominance, 3 liking
    valence_labels = labels[:, 0] >= np.median(labels[:,0]) # valence labels
    # convert true to 1, false to 0
    valence_labels = valence_labels.astype(int) 
    arousal_labels = labels[:, 1] >= np.median(labels[:,1]) # arousal labels
    arousal_labels = arousal_labels.astype(int)
    arousal, valence = [], []
    for i in range(len(valence_labels)):
        for j in range(0, sample_size):
            valence = valence + [valence_labels[i]]
            arousal = arousal + [arousal_labels[i]]
    return np.array(valence), np.array(arousal)

if __name__ == '__main__':
    for n in range(1, NUM_SUBJECTS+1): 
        filename = ''
        if n < 10:
            filename += '0'
        filename += str(n)
        print("processing: ", filename, "......")
        file_path = DATA_PATH + "s" + filename + ".dat"
        trial = read_eeg_signal_from_file(file_path)
        labels = trial['labels']
        data = trial['data']
        # eeg features
        fft_data, psd, diff_entropy = get_eeg_features(data)
        # peripheral features
        peripheral_data = get_peripheral_features(data)
        # labels
        valence_labels, arousal_labels = get_labels(labels)
        # save features as npz
        np.savez_compressed(SAVE_PATH + filename + ".npz", fft=fft_data, psd=psd, diff_entropy=diff_entropy, peripheral_data=peripheral_data, valence_labels=valence_labels, arousal_labels=arousal_labels)        
    print("Done")
