# classify a subject eeg data from deap using svm
import numpy as np
from sklearn.svm import SVC
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import MinMaxScaler
from sklearn.metrics import accuracy_score

DATA_PATH = "C://Users/YOGA/Desktop/phd/datasets/deap_archive/psd/"
DATA_PATH2 = "C://Users/YOGA/Desktop/phd/datasets/deap_archive/1d/"
DATA_PATH3 = "C://Users/YOGA/Desktop/phd/datasets/deap_archive/fft/"
NUM_SUBJECTS = 32

def load_test_data(subject_id):
    if subject_id < 10:
        filename = "0" + str(subject_id)
    # read npz file
    data = np.load(DATA_PATH + filename + ".npz")
    psd = data['psd']
    diff_entropy = data['diff_entropy']
    peripheral_data = data['peripheral_data']
    valence_labels = data['valence_labels']
    arousal_labels = data['arousal_labels']
    return psd, diff_entropy, peripheral_data, valence_labels, arousal_labels

def load_fft_data(subject_id):
    if subject_id < 10:
        filename = "0" + str(subject_id)
    # read npz file
    data = np.load(DATA_PATH3 + filename + ".npz")
    fft = data['fft']
    psd = data['psd']
    diff_entropy = data['diff_entropy']
    peripheral_data = data['peripheral_data']
    valence_labels = data['valence_labels']
    arousal_labels = data['arousal_labels']
    return fft, psd, diff_entropy, peripheral_data, valence_labels, arousal_labels

def load_train_data(subject_id):
    psd_train, diff_entropy_train, peripheral_data_train, valence_labels_train, arousal_labels_train  = None, None, None, None, None
    for n in range(1, NUM_SUBJECTS+1): 
        filename = ''
        if n < 10:
            filename += '0'
        filename += str(n)
        if n != subject_id:
            # read npz file
            data = np.load(DATA_PATH + filename + ".npz")
            psd = data['psd']
            diff_entropy = data['diff_entropy']
            peripheral_data = data['peripheral_data']
            valence_labels = data['valence_labels']
            arousal_labels = data['arousal_labels']
            if psd_train is None:
                psd_train = psd
                diff_entropy_train = diff_entropy
                peripheral_data_train = peripheral_data
                valence_labels_train = valence_labels
                arousal_labels_train = arousal_labels
            else:
                psd_train = np.concatenate((psd_train, psd), axis=0)
                diff_entropy_train = np.concatenate((diff_entropy_train, diff_entropy), axis=0)
                peripheral_data_train = np.concatenate((peripheral_data_train, peripheral_data), axis=0)
                valence_labels_train = np.concatenate((valence_labels_train, valence_labels), axis=0)
                arousal_labels_train = np.concatenate((arousal_labels_train, arousal_labels), axis=0)
    return psd_train, diff_entropy_train, peripheral_data_train, valence_labels_train, arousal_labels_train  

def load_test_data2(subject_id):
    #"DE_s02.dat"
    if subject_id < 10:
        filename = "0" + str(subject_id)
    filename = DATA_PATH2 + "DE_s" + filename + ".dat.npz"
    # read npz file
    data = np.load(filename)
    base_DE = data['base_data']
    trial_DE = data['data']
    valence_labels = data['valence_labels']
    arousal_labels = data['arousal_labels']
    return base_DE, trial_DE, valence_labels, arousal_labels

def load_train_data2(subject_id):
    base_DE_train, trial_DE_train, valence_labels_train, arousal_labels_train = None, None, None, None
    for n in range(1, NUM_SUBJECTS+1): 
        filename = ''
        if n < 10:
            filename += '0'
        filename += str(n)
        if n != subject_id:
            # read npz file
            filename = DATA_PATH2 + "DE_s" + filename + ".dat.npz"
            data = np.load(filename)
            base_DE = data['base_data']
            trial_DE = data['data']
            valence_labels = data['valence_labels']
            arousal_labels = data['arousal_labels']
            if base_DE_train is None:
                base_DE_train = base_DE
                trial_DE_train = trial_DE
                valence_labels_train = valence_labels
                arousal_labels_train = arousal_labels
            else:
                base_DE_train = np.concatenate((base_DE_train, base_DE), axis=0)
                trial_DE_train = np.concatenate((trial_DE_train, trial_DE), axis=0)
                valence_labels_train = np.concatenate((valence_labels_train, valence_labels), axis=0)
                arousal_labels_train = np.concatenate((arousal_labels_train, arousal_labels), axis=0)
    return base_DE_train, trial_DE_train, valence_labels_train, arousal_labels_train

def calc_trial_peripheral(data):
    peripheral_data = np.zeros((len(data)*120, 8*8))
    for i in range(len(data)):
        for j in range(0, 120):
            peripheral_data[i*120+j] = data[i]
    return peripheral_data

def calculate_accuracy(data, labels):
    #model = SVC(C=2, kernel = 'rbf', degree=3)
    model = RandomForestClassifier(n_estimators=100, random_state=1)
    # split data
    data_train, data_test, labels_train, labels_test = train_test_split(data, labels, test_size=0.2, random_state=42)
    # scale features
    scaler = MinMaxScaler()
    scaler_model = scaler.fit(data_train)
    data_train = scaler_model.transform(data_train)
    # train svm
    model.fit(data_train, labels_train)
    # scale test data
    data_test = scaler_model.transform(data_test)
    # test svm
    accuracy = model.score(data_test, labels_test)
    return accuracy

def calculate_accuracy2(trainX, trainY, testX, testY):
    #model = SVC(C=2, kernel = 'rbf', degree=3)
    model = RandomForestClassifier(n_estimators=100, random_state=1)
    # scale features
    scaler = MinMaxScaler()
    trainX = scaler.fit_transform(trainX)
    # train svm
    model.fit(trainX, trainY)
    # scale test data
    scaler2 = MinMaxScaler()
    testX = scaler2.fit_transform(testX)
    # test svm
    accuracy = model.score(testX, testY)
    return accuracy

if __name__ == '__main__':
    train_id , test_id = 3, 4
    fft, psd, diff_entropy, peripheral_data, valence_labels, arousal_labels = load_fft_data(train_id)
    peripheral_data[~np.isfinite(peripheral_data)] = 0
    fft2, psd2, diff_entropy2, peripheral_data2, valence_labels2, arousal_labels2 = load_fft_data(test_id)
    peripheral_data2[~np.isfinite(peripheral_data2)] = 0
    print("fft - valence: ", calculate_accuracy2(fft, valence_labels, fft2, valence_labels2))
    print("fft - arousal: ", calculate_accuracy2(fft, arousal_labels, fft2, arousal_labels2))
    print("psd - valence: ", calculate_accuracy2(psd, valence_labels, psd2, valence_labels2))
    print("psd - arousal: ", calculate_accuracy2(psd, arousal_labels, psd2, arousal_labels2))
    print("diff_entropy - valence: ", calculate_accuracy2(diff_entropy, valence_labels, diff_entropy2, valence_labels2))
    print("diff_entropy - arousal: ", calculate_accuracy2(diff_entropy, arousal_labels, diff_entropy2, arousal_labels2))
    print("peripheral_data - valence: ", calculate_accuracy2(peripheral_data, valence_labels, peripheral_data2, valence_labels2))
    print("peripheral_data - arousal: ", calculate_accuracy2(peripheral_data, arousal_labels, peripheral_data2, arousal_labels2))
    # concat fft with peripheral data
    fft = np.concatenate((fft, peripheral_data), axis=1)
    fft2 = np.concatenate((fft2, peripheral_data2), axis=1)
    print("fft + peripheral_data - valence: ", calculate_accuracy2(fft, valence_labels, fft2, valence_labels2))
    print("fft + peripheral_data - arousal: ", calculate_accuracy2(fft, arousal_labels, fft2, arousal_labels2))
    # concat psd with peripheral data
    psd = np.concatenate((psd, peripheral_data), axis=1)
    psd2 = np.concatenate((psd2, peripheral_data2), axis=1)
    print("psd + peripheral_data - valence: ", calculate_accuracy2(psd, valence_labels, psd2, valence_labels2))
    print("psd + peripheral_data - arousal: ", calculate_accuracy2(psd, arousal_labels, psd2, arousal_labels2))
    # concat diff_entropy with peripheral data
    diff_entropy = np.concatenate((diff_entropy, peripheral_data), axis=1)
    diff_entropy2 = np.concatenate((diff_entropy2, peripheral_data2), axis=1)
    print("diff_entropy + peripheral_data - valence: ", calculate_accuracy2(diff_entropy, valence_labels, diff_entropy2, valence_labels2))
    print("diff_entropy + peripheral_data - arousal: ", calculate_accuracy2(diff_entropy, arousal_labels, diff_entropy2, arousal_labels2))

def test():
    test_id = 2
    # load data2
    base_data, trial_data, valence_labels2, arousal_labels2 = load_train_data2(test_id)
    # load data
    psd, diff_entropy, peripheral_data, valence_labels, arousal_labels = load_train_data(test_id)
    # reshape data for svm
    psd = np.reshape(psd, (len(psd), 32*4))
    diff_entropy = np.reshape(diff_entropy, (len(diff_entropy), 32*4))
    peripheral_data = np.reshape(peripheral_data, (len(peripheral_data), 8*8))

    print("psd - valence: ", calculate_accuracy(psd, valence_labels))
    print("psd - arousal: ", calculate_accuracy(psd, arousal_labels))
    print("diff_entropy - valence: ", calculate_accuracy(diff_entropy, valence_labels))
    print("diff_entropy - arousal: ", calculate_accuracy(diff_entropy, arousal_labels))
    print("peripheral_data - valence: ", calculate_accuracy(peripheral_data, valence_labels))
    print("peripheral_data - arousal: ", calculate_accuracy(peripheral_data, arousal_labels))
    # concat vertical psd and peripheral data
    psd_peripheral_data = np.concatenate((psd, peripheral_data), axis=1)
    print("psd + peripheral_data - valence: ", calculate_accuracy(psd_peripheral_data, valence_labels))
    print("psd + peripheral_data - arousal: ", calculate_accuracy(psd_peripheral_data, arousal_labels))
    # concat vertical diff_entropy and peripheral data
    diff_entropy_peripheral_data = np.concatenate((diff_entropy, peripheral_data), axis=1)
    print("diff_entropy + peripheral_data - valence: ", calculate_accuracy(diff_entropy_peripheral_data, valence_labels))
    print("diff_entropy + peripheral_data - arousal: ", calculate_accuracy(diff_entropy_peripheral_data, arousal_labels))

    print("base_data - valence: ", calculate_accuracy(base_data, valence_labels))
    print("base_data - arousal: ", calculate_accuracy(base_data, arousal_labels))
    # concat vertical base_data and peripheral data
    base_data_peripheral_data = np.concatenate((base_data, peripheral_data), axis=1)
    print("base_data + peripheral_data - valence: ", calculate_accuracy(base_data_peripheral_data, valence_labels))
    print("base_data + peripheral_data - arousal: ", calculate_accuracy(base_data_peripheral_data, arousal_labels))
    print(".........................")
    # load test subject
    test_base_data, test_trial_data, test_valence_labels2, test_arousal_labels2 = load_test_data2(test_id)
    test_psd, test_diff_entropy, test_peripheral_data, test_valence_labels, test_arousal_labels = load_test_data(test_id)
    # reshape data for svm
    test_psd = np.reshape(test_psd, (len(test_psd), 32*4))
    test_diff_entropy = np.reshape(test_diff_entropy, (len(test_diff_entropy), 32*4))
    test_peripheral_data = np.reshape(test_peripheral_data, (len(test_peripheral_data), 8*8))
    test_peripheral_data2 = calc_trial_peripheral(test_peripheral_data)

    print("psd - valence: ", calculate_accuracy2(psd, valence_labels, test_psd, test_valence_labels))
    print("psd - arousal: ", calculate_accuracy2(psd, arousal_labels, test_psd, test_arousal_labels))
    print("diff_entropy - valence: ", calculate_accuracy2(diff_entropy, valence_labels, test_diff_entropy, test_valence_labels))
    print("diff_entropy - arousal: ", calculate_accuracy2(diff_entropy, arousal_labels, test_diff_entropy, test_arousal_labels))
    print("peripheral_data - valence: ", calculate_accuracy2(peripheral_data, valence_labels, test_peripheral_data, test_valence_labels))
    print("peripheral_data - arousal: ", calculate_accuracy2(peripheral_data, arousal_labels, test_peripheral_data, test_arousal_labels))

    # concat vertical psd and peripheral data
    test_psd_peripheral_data = np.concatenate((test_psd, test_peripheral_data), axis=1)
    print("psd + peripheral_data - valence: ", calculate_accuracy2(psd_peripheral_data, valence_labels, test_psd_peripheral_data, test_valence_labels))
    print("psd + peripheral_data - arousal: ", calculate_accuracy2(psd_peripheral_data, arousal_labels, test_psd_peripheral_data, test_arousal_labels))
    # concat vertical diff_entropy and peripheral data
    test_diff_entropy_peripheral_data = np.concatenate((test_diff_entropy, test_peripheral_data), axis=1)
    print("diff_entropy + peripheral_data - valence: ", calculate_accuracy2(diff_entropy_peripheral_data, valence_labels, test_diff_entropy_peripheral_data, test_valence_labels))
    print("diff_entropy + peripheral_data - arousal: ", calculate_accuracy2(diff_entropy_peripheral_data, arousal_labels, test_diff_entropy_peripheral_data, test_arousal_labels))

    print("base_data - valence: ", calculate_accuracy2(base_data, valence_labels, test_base_data, test_valence_labels))
    print("base_data - arousal: ", calculate_accuracy2(base_data, arousal_labels, test_base_data, test_arousal_labels))
    # concat vertical base_data and peripheral data
    test_base_data_peripheral_data = np.concatenate((test_base_data, test_peripheral_data), axis=1)
    print("base_data + peripheral_data - valence: ", calculate_accuracy2(base_data_peripheral_data, valence_labels, test_base_data_peripheral_data, test_valence_labels))
    print("base_data + peripheral_data - arousal: ", calculate_accuracy2(base_data_peripheral_data, arousal_labels, test_base_data_peripheral_data, test_arousal_labels))



