import numpy as np
import pickle
import scipy.io as sio
import os
from sqlalchemy import null
import torch
import pandas as pd
from torch.utils.data import Dataset
from torch.utils.data import TensorDataset
from torch.utils.data import DataLoader
from sklearn.preprocessing import MinMaxScaler

NUMBER_OF_SESSIONS = 10
DATA_PATH = "allData.csv"
Data = pd.read_csv(DATA_PATH).dropna()
Data['label'] = np.where(Data['Valence'] > np.median(Data['Valence']), 1, 0) #"Valence","Arousal"
SUBJECTS = Data['subject'].unique()
STIMULI = Data['stimuli'].unique()

NUMBER_OF_SUBJECTS = SUBJECTS.size

eeg_cols = ["delta.AF3","theta.AF3","alpha.AF3","beta.AF3","gamma.AF3","delta.AF4","theta.AF4","alpha.AF4","beta.AF4","gamma.AF4","delta.Fp1","theta.Fp1","alpha.Fp1","beta.Fp1","gamma.Fp1","delta.Fp2","theta.Fp2","alpha.Fp2","beta.Fp2","gamma.Fp2","delta.AF7","theta.AF7","alpha.AF7","beta.AF7","gamma.AF7","delta.AF8","theta.AF8","alpha.AF8","beta.AF8","gamma.AF8","attention","relaxation","asymmetry","leftActivity","rightActivity"]
peripheral_cols = ["X_min","Y_min","Z_min","X_mean","Y_mean","Z_mean","X_max","Y_max","Z_max","X_median","Y_median","Z_median","X_sd","Y_sd","Z_sd"]

def load_subject_transfer_data(subject_id, test_session_id):
    subject_data = Data[Data['subject'] == SUBJECTS[subject_id-1]]
    other_data = Data[Data['subject'] != SUBJECTS[subject_id-1]]
    scaler = MinMaxScaler()
    subject_data[eeg_cols] = scaler.fit_transform(subject_data[eeg_cols])
    subject_data[peripheral_cols] = scaler.fit_transform(subject_data[peripheral_cols])
    other_data[eeg_cols] = scaler.fit_transform(other_data[eeg_cols])
    other_data[peripheral_cols] = scaler.fit_transform(other_data[peripheral_cols])
    nrows = subject_data.shape[0]
    # For CIS Score
    gen_data_size = int(nrows*(test_session_id/NUMBER_OF_SESSIONS))
    gen_data = subject_data.iloc[0:gen_data_size]
    svc_data = subject_data.iloc[gen_data_size:]

    EEG_gen_test, EEG_gen_label = gen_data[eeg_cols].values, gen_data['label'].values
    EYE_gen_test, EYE_gen_label = gen_data[peripheral_cols].values, gen_data['label'].values
    EEG_svc_test, EEG_svc_label = svc_data[eeg_cols].values, svc_data['label'].values
    EYE_svc_test, EYE_svc_label = svc_data[peripheral_cols].values, svc_data['label'].values

    EEG_data, EEG_label = other_data[eeg_cols].values, other_data['label'].values
    EYE_data, EYE_label = other_data[peripheral_cols].values, other_data['label'].values

    return EEG_data, EEG_label, EYE_data, EYE_label, EEG_gen_test, EEG_gen_label, EYE_gen_test, EYE_gen_label, EEG_svc_test, EEG_svc_label, EYE_svc_test, EYE_svc_label

def load_subject_data(subject_id):
    subject_data = Data[Data['subject'] == SUBJECTS[subject_id-1]]
    scaler = MinMaxScaler()
    subject_data[eeg_cols] = scaler.fit_transform(subject_data[eeg_cols])
    subject_data[peripheral_cols] = scaler.fit_transform(subject_data[peripheral_cols])

    EEG_test, EEG_label = subject_data[eeg_cols].values, subject_data['label'].values
    EYE_test, EYE_label = subject_data[peripheral_cols].values, subject_data['label'].values

    return EEG_test, EEG_label, EYE_test, EYE_label

if __name__ == '__main__':
    EEG_data, EEG_label, EYE_data, EYE_label, EEG_gen_test, EEG_gen_label, EYE_gen_test, EYE_gen_label, EEG_svc_test, EEG_svc_label, EYE_svc_test, EYE_svc_label = load_subject_transfer_data(1, 1)
    print("EEG:",EEG_data.shape,"EYE:",EYE_data.shape," label:",EEG_label.shape)
    print("NUMBER_OF_SUBJECTS:",NUMBER_OF_SUBJECTS)
    print("eeg_cols:",len(eeg_cols), " peripheral_cols:",len(peripheral_cols))
    #train_dataset = TensorDataset(torch.FloatTensor(EEG_data), torch.FloatTensor(EYE_data), torch.LongTensor(EEG_label))
    #train_dataloader = DataLoader(train_dataset, batch_size = 20, shuffle = True)
