# read mp4 file and extract features

import os
import cv2
import numpy as np
import time
import pandas as pd

# Parameters for Lucas-Kanade optical flow
lk_params = dict(winSize=(15, 15), maxLevel=2, criteria=(cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, 10, 0.03))

# read video files from directory 'DENEY-DATALAR'
def readVideoFiles():
    videoFiles = []
    for root, dirs, files in os.walk("DENEY-DATALAR"):
        for file in files:
            if file.endswith(".mp4"):
                videoFiles.append(os.path.join(root, file))
    return videoFiles

def getVideoFeatures(video_file, videoDate, videoTime):
    # Open the video file
    cap = cv2.VideoCapture(video_file)
    # Get video properties
    fps = cap.get(cv2.CAP_PROP_FPS)
    frame_count = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
    duration = frame_count/fps
    # add elapsed time to video time as seconds
    videoTime = videoTime.split(":")
    videoTime = int(videoTime[0]) * 3600 + int(videoTime[1]) * 60 + int(videoTime[2])
    filename = video_file.split("\\")[1][:2]+'_videoFeatures.csv'
    # Initialize variables
    old_frame = None
    old_points = None
    while True:
        # Read the current frame from the video
        ret, frame = cap.read()

        if not ret:
            break  # Break the loop when the video ends
        
        try:
            # Convert frame to grayscale
            gray_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)

            # If it's the first frame, initialize the previous frame and points
            if old_frame is None:
                old_frame = gray_frame
                p0 = cv2.goodFeaturesToTrack(old_frame, mask=None, maxCorners=100, qualityLevel=0.3, minDistance=7, blockSize=7)
                old_points = p0

            # Calculate optical flow using Lucas-Kanade algorithm
            p1, st, err = cv2.calcOpticalFlowPyrLK(old_frame, gray_frame, old_points, None, **lk_params)
            
            if p1 is None:
                p0 = cv2.goodFeaturesToTrack(old_frame, mask=None, maxCorners=100, qualityLevel=0.3, minDistance=7, blockSize=7)
                old_points = p0
                p1, st, err = cv2.calcOpticalFlowPyrLK(old_frame, gray_frame, old_points, None, **lk_params)
        except:
            continue
        # Select good points
        good_new = p1[st == 1]
        good_old = old_points[st == 1]

        # Calculate the mean movement of points
        if len(good_new) > 0 and len(good_old) > 0:
            mean_movement = np.mean(good_new - good_old, axis=0)
            dx, dy = mean_movement.ravel()
            dz = np.sqrt(dx ** 2 + dy ** 2)

            # Calculate elapsed time based on frame index and FPS
            current_frame = cap.get(cv2.CAP_PROP_POS_FRAMES)
            elapsed_time = current_frame / fps

            elapsed_time = elapsed_time + videoTime
            # convert elapsed time to datetime
            elapsed_time = time.strftime('%H:%M:%S', time.gmtime(elapsed_time))
            # write to csv file
            df = pd.DataFrame([[videoDate, elapsed_time, dx, dy, dz]], columns=['Date', 'Time', 'X', 'Y', 'Z'])
            df.to_csv(filename, mode='a', header=False, index=False)

        # Update previous frame and points
        old_frame = gray_frame.copy()
        old_points = good_new.reshape(-1, 1, 2)
    # Release resources
    cap.release() 

def main():
    videoFiles = readVideoFiles()
    for videoFile in videoFiles[-2:]:
        videoDate = videoFile.split(" ")[0].split("\\")[1].split("_")[1]
        videoTime = videoFile.split(" ")[1].split(".")[0]
        # change - to : in time
        videoTime = videoTime.replace("-", ":")
        print(videoFile, videoDate, videoTime)
        getVideoFeatures(videoFile, videoDate, videoTime)

if __name__ == "__main__":
    main()
