import numpy as np
import pickle
import scipy.io as sio
import os
from sqlalchemy import null
import torch
from torch.utils.data import Dataset
from torch.utils.data import TensorDataset
from torch.utils.data import DataLoader
from sklearn.preprocessing import MinMaxScaler

NUMBER_OF_SUBJECTS = 16

DATASET_EEG_PATH = "../SEED-V/EEG_DE_features"
DATASET_EYE_PATH = "../SEED-V/Eye_movement_features"

def load_data_eeg(idx, start, end):
    EEG_filename = str(idx)+"_123.npz"
    EEG_npz = np.load(os.path.join(DATASET_EEG_PATH, EEG_filename))
    EEG_data = pickle.loads(EEG_npz['data']) # [45][event,310]
    EEG_label = pickle.loads(EEG_npz['label'])
    i_0 = start-1
    ses1_X_train = EEG_data[i_0]
    ses1_y_train = EEG_label[i_0]
    for session in range(start,  end):
        ses1_X_train = np.vstack((ses1_X_train, EEG_data[session] ))
        ses1_y_train = np.concatenate((ses1_y_train, EEG_label[session]), axis = 0)

    scaler = MinMaxScaler()
    ses1_X_train = scaler.fit(ses1_X_train).transform(ses1_X_train)
    return ses1_X_train, ses1_y_train

def load_data_eye(idx, start, end):
    EYE_filename = str(idx)+"_123.npz"
    EYE_npz = np.load(os.path.join(DATASET_EYE_PATH, EYE_filename))
    EYE_data = pickle.loads(EYE_npz['data']) # [45][event,33]
    EYE_label = pickle.loads(EYE_npz['label'])
    i_0 = start-1
    ses1_X_train = EYE_data[i_0]
    ses1_y_train = EYE_label[i_0]
    for session in range(start, end):
        ses1_X_train = np.vstack((ses1_X_train, EYE_data[session] ))
        ses1_y_train = np.concatenate((ses1_y_train, EYE_label[session]), axis = 0)

    scaler = MinMaxScaler()
    ses1_X_train = scaler.fit(ses1_X_train).transform(ses1_X_train)
    return ses1_X_train, ses1_y_train

def load_session_transfer_data(subject_id, test_session_id):
    if test_session_id == 1:
        data_start = 16
        data_end = 45
        gen_start = 1
        gen_end = 10
        test_start = 11
        test_end = 15
    elif test_session_id == 2:
        data_start = 1
        data_end = 15
        gen_start = 16
        gen_end = 25
        test_start = 26
        test_end = 30
    elif test_session_id == 3:
        data_start = 1
        data_end = 30
        gen_start = 31
        gen_end = 40
        test_start = 41
        test_end = 45

    EEG_data, EEG_label = load_data_eeg(subject_id, data_start, data_end)
    EYE_data, EYE_label = load_data_eye(subject_id, data_start, data_end)
    EEG_gen_test, EEG_gen_label = load_data_eeg(subject_id, gen_start, gen_end)
    EYE_gen_test, EYE_gen_label = load_data_eye(subject_id, gen_start, gen_end)
    EEG_svc_test, EEG_svc_label = load_data_eeg(subject_id, test_start, test_end)
    EYE_svc_test, EYE_svc_label = load_data_eye(subject_id, test_start, test_end)

    if test_session_id == 2:
        EEG_data2, EEG_label2 = load_data_eeg(subject_id, 31, 45)
        EYE_data2, EYE_label2 = load_data_eye(subject_id, 31, 45)
        EEG_data = np.vstack((EEG_data, EEG_data2))
        EEG_label = np.concatenate((EEG_label, EEG_label2), axis = 0)
        EYE_data = np.vstack((EYE_data, EYE_data2))
        EYE_label = np.concatenate((EYE_label, EYE_label2), axis = 0)

    return EEG_data, EEG_label, EYE_data, EYE_label, EEG_gen_test, EEG_gen_label, EYE_gen_test, EYE_gen_label, EEG_svc_test, EEG_svc_label, EYE_svc_test, EYE_svc_label

def load_subject_transfer_data(subject_id, test_session_id):
    EEG_data, EEG_label, EYE_data, EYE_label = null, null, null, null
    for i in range(1, NUMBER_OF_SUBJECTS+1):
        if i == subject_id:
            if test_session_id == 1:
                gen_start = 16
                gen_end = 45
                test_start = 1
                test_end = 15
            elif test_session_id == 2:
                gen_start = 1
                gen_end = 15
                test_start = 16
                test_end = 30
            elif test_session_id == 3:
                gen_start = 1
                gen_end = 30
                test_start = 31
                test_end = 45
            EEG_gen_test, EEG_gen_label = load_data_eeg(subject_id, gen_start, gen_end)
            EYE_gen_test, EYE_gen_label = load_data_eye(subject_id, gen_start, gen_end)
            EEG_svc_test, EEG_svc_label = load_data_eeg(subject_id, test_start, test_end)
            EYE_svc_test, EYE_svc_label = load_data_eye(subject_id, test_start, test_end)
            if test_session_id == 2:
                EEG_gen_test2, EEG_gen_label2 = load_data_eeg(subject_id, 31, 45)
                EYE_gen_test2, EYE_gen_label2 = load_data_eye(subject_id, 31, 45)
                EEG_gen_test = np.vstack((EEG_gen_test, EEG_gen_test2))
                EEG_gen_label = np.concatenate((EEG_gen_label, EEG_gen_label2), axis = 0)
                EYE_gen_test = np.vstack((EYE_gen_test, EYE_gen_test2))
                EYE_gen_label = np.concatenate((EYE_gen_label, EYE_gen_label2), axis = 0)
        else:
            if EEG_data is null:
                EEG_data, EEG_label = load_data_eeg(i, 1, 45)
                EYE_data, EYE_label = load_data_eye(i, 1, 45)
            else:
                EEG_data2, EEG_label2 = load_data_eeg(i, 1, 45)
                EYE_data2, EYE_label2 = load_data_eye(i, 1, 45)
                EEG_data = np.vstack((EEG_data, EEG_data2))
                EEG_label = np.concatenate((EEG_label, EEG_label2), axis = 0)
                EYE_data = np.vstack((EYE_data, EYE_data2))
                EYE_label = np.concatenate((EYE_label, EYE_label2), axis = 0)

    return EEG_data, EEG_label, EYE_data, EYE_label, EEG_gen_test, EEG_gen_label, EYE_gen_test, EYE_gen_label, EEG_svc_test, EEG_svc_label, EYE_svc_test, EYE_svc_label

if __name__ == '__main__':
    EEG_data, EEG_label = load_data_eeg(1, 1, 30)
    EYE_data, EYE_label = load_data_eye(1, 1, 30)
    print((EEG_label==EYE_label).all())
    print("EEG:",EEG_data.shape,"EYE:",EYE_data.shape," label:",EEG_label.shape)
    #train_dataset = TensorDataset(torch.FloatTensor(EEG_data), torch.FloatTensor(EYE_data), torch.LongTensor(EEG_label))
    #train_dataloader = DataLoader(train_dataset, batch_size = 20, shuffle = True)
